;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;
; UDG animation
;
;================================

ORG 4084

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@Class6=0D92			;Expression evaluator, if value>255 then error B
@StackA=0C02			;Move results to A
@FirstFrame=4004		;First frame address (Ramtop)
@LastK=4025			;Key press value, 16421
@NxtLine=4029			;Address of next program line to be executed

		JR Player
;================================
; Vars
;================================
@LastFrame
		00 00		;16518, Address of last frame
@Speed
		00		;16520, Speed
@Lups
		00		;16521, Loop Count
@Framz
		00		;16522, Frame count
@FNum
		00		;16523, Frame countdown
@InOut
		00		;16524, 0=Forward, 1=Reverse
;================================
@ErrorP
	LD A,1E
	LD I,A
	RST 08
	18			;Error P, parameter missing, Code-29
;================================
@GetPar
	RST 20			;Get next character
	CALL Class6		;Evaluate expression 
	CALL StackA		;A=Frame count
	RET
;================================
; Animation Player
;
; PRINT USR PLAY;[Frames],[Speed],[Direction],[Loop Count],[Invert]
;
;================================
@Player
	CALL GetPar		;Evaluate [Frames] expression
	LD (Framz),A		;Set frame count constant
	CALL GetPar		;Evaluate [Speed] expression
	INC A
	LD (Speed),A		;Set speed
	CALL GetPar		;Evaluate [Direction] expression
	LD (InOut),A		;Set InOut
	CALL GetPar		;Evaluate [Loop Counter] expression
	LD (Lups),A		;Set loop counter
	CALL GetPar		;Evaluate [Invert] expression
	AND A
	CALL NZ SwapBW
;================================
	LD A,(Framz)		;Calculate last frame address
	LD B,A
	LD HL,(FirstFrame)	;
	LD B,A			;B=frame count
	LD DE,0400		;Byte count/frame
@GetLast
	ADD HL,DE
	DJNZ GetLast
	LD (LastFrame),HL	;Save lastframe address
;================================
@Reset
	LD A,(InOut)		;Rotation setup
	AND A			;Zero test
	LD HL,(FirstFrame)	;Get firstframe address
	JR Z +3			;Jump if 0
	LD HL,(LastFrame)	;Get last frame address
	LD (Frame0+1),HL	;Set starting frame
	LD A,(Framz)		;Get frame count
;================================
@FLoop
	LD (Fnum),A		;Current Frame
@Frame0
	LD HL,0000		;Starting frame address
;================================
; Load UDG definition to Bank 1 or 2
;================================
@NextFrame
	CALL Rotate		;Rotate forward/reverse
	EXX
	LD HL,Speed		;Speed address
	LD A,(HL)		;Get delay between frames
	LD B,A			;Set delay counter
@Delay1
	LD C,A			;Set delay counter
@Delay2
	DEC C
	JR NZ Delay2
	DJNZ Delay1
;================================
; Sort out the key presses
;================================
	CALL Keyscan		;Page keyboard
	CP FF
	JR Z Continue		;Jump if no keypress
@KeySort
	LD HL,Speed		;Speed address
@KeyA
	CP "A"			;Faster?
	JR NZ KeyZ		;No, move on
	LD A,(HL)		;Speed value
	CP 01			;1 test
	JR Z Continue		;Jump if 1
	DEC (HL)		;Decrease delay counter
	JR Continue		;Continue animation
@KeyZ
	CP "Z"			;Slower?
	JR NZ KeyI		;No, move on
	INC (HL)		;Increase delay counter
	JR Continue		;Continue animation
@KeyI
	CP "S"			;Swap Paper/Ink?
	JR NZ KeyR		;No, move on
	CALL Wait
	CALL SwapBW
	JR Continue
@KeyR
	CP "R"			;Reverse?
	JR NZ KeyP		;No, move on
	CALL Wait
	LD A,(InOut)
	AND A
	LD A,01
	JR Z +1
	XOR A
	LD (InOut),A
	JR Reset		;Restart animation
@KeyP
	CP "P"			;Pulse?
	JR NZ Key1		;No, move on
	CALL Wait
	LD HL,+200		;Line 200, Pulse routine
	JR SetLine
@Key1
	CP "1"			;Next Paper color?
	JR NZ Key2		;No, move on
	LD HL,+300		;Line 300, Next Paper routine
	JR SetLine
@Key2
	CP "2"			;Next Ink color?
	JR NZ Key3		;No, move on
	LD HL,+400		;Line 400, Next Ink routine
	JR SetLine
@Key3
	CP "3"			;Random Paper/Ink?
	JR NZ KeyQ		;No, move on
	LD HL,+500		;Line 500, Next Ink routine
@SetLine
	CALL Wait
	CALL LAddr
	LD (NxtLine),HL
@End
	RST 08
	FF
@KeyQ
	CP "Q"			;Quit?
	JR NZ Continue		;No, move on
	LD HL,+9400		;Line 9400, Reload main menu 
	JR SetLine
@Continue
	EXX
	LD A,(FNum)		;Get frame count
	DEC A			;Decrease frame count
	LD (FNum),A		;Store count
	JP NZ NextFrame
	LD A,(Lups)		;Get loop count
	AND A
	JR Z ResetFramz
	DEC A			;Decrease loop count
	JR Z End		;Jump if 0
	LD (Lups),A		;Store loop count
@ResetFramz
	LD A,(Framz)		;Reset frame counter
	JP FLoop
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress , D=FF if no key
	LD A,D			;A=FF if no key press
	INC D 			;D=0 if no key
	RET Z 			;No keypress
	CALL Decode		;Decode routine converts BC to ZX81 keycodes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Wait for key release
;================================
@Wait
	LD A,(LastK)		;Get keyboard status
	CP FF			;Key pressed?
	JR NZ Wait		;Jump if keypress
	RET
;================================
; Swap Paper/Ink
;================================
@Swap
	CALL Wait
	CALL SwapBW
	JR Continue
@SwapBW
	LD A,(Framz)		;Get frame count
	LD B,A			;Set DJNZ counter
	LD HL,(FirstFrame)	;Start of data
@SwapOuter
	LD DE,0400		;Byte count/frame
@SwapInner
	LD A,(HL)		;CPL chr
	CPL
	LD (HL),A
	INC HL			;Next data
	LD A,D			;Decrease counter
	OR E
	DEC DE
	JR NZ SwapInner
	DJNZ SwapOuter
	RET
;================================
; Forward/Reverse Rotation
;================================
@Rotate
	LD A,I			;"Double Buffer" IReg for
	CP 21			;Smoother animation.
	JR NZ +4
	LD A,24
	JR +2
	LD A,20
	PUSH AF			;PUSH IReg
	LD BC,0400		;Byte count
	LD D,A			;Set destination, 2000 or 2400
	LD E,C
	LD A,(InOut)		;Get InOut
	AND A			;Zero test
	JR NZ Reverse		;Jump if not 0
	LDIR			;Copy pattern to UDG table
@IReg
	POP AF			;POP IReg
	INC A			;A=21 or 25
	LD I,A			;Activate Bank 1/2
	RET
@Reverse
	SBC HL,BC		;Move up in data file
	PUSH HL			;PUSH pattern addr
	LDIR			;Copy pattern to UDG table
	POP HL			;POP pattern address
	JR IReg